#========================================================================================
# package      DE_EPAGES::Test::t::unit::MockifyServlet
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for MockifyServlet
#========================================================================================
package DE_EPAGES::Test::t::unit::MockifyServlet;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_Servlet_object();
    $self->test_Servlet_WrongObject();
    $self->test_addVar();
    $self->test_addVar_wrongType();
    $self->test_addVar_wrongValue();
    $self->test_addVars();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_Servlet_object {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopObject = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $ShopObject );
    my $Servlet = $MockifyServlet->getMockObject();

    my $Result = $Servlet->object();

    isa_ok( $Result, 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - tests if the servlet object is the expected one" );
}
#----------------------------------------------------------------------------------------
sub test_Servlet_WrongObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $NotEpagesObject = bless( {}, 'SomePerlObject' );

    throws_ok( sub { DE_EPAGES::Test::Mock::MockifyServlet->new( $NotEpagesObject ) },
               qr/ObjectInServlet is not an instance of DE_EPAGES::Core::API::Object::BaseObject/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
               "$SubTestName - test the Error if the object is not an epages object"
     );
}
#----------------------------------------------------------------------------------------
sub test_addVar {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopObject = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $ShopObject );
    $MockifyServlet->addVar( 'Key1', 'Value1' );
    $MockifyServlet->addVar( 'Key2', 'Value2', 'string' );
    $MockifyServlet->addVar( 'Key3', 'Value3', {'string'=>'abcd'} );
    my $Servlet = $MockifyServlet->getMockObject();

    my $ResultKey1 = $Servlet->vars( 'Key1' );

    my $ResultKey2 = $Servlet->vars( 'Key2' );
    my $ResultKey2OnChange = $Servlet->vars( 'Key2', 'NewValue' );
    my $ResultKey2AfterChange = $Servlet->vars( 'Key2' );

    my $ResultKey3 = $Servlet->vars( 'Key3' );
    my $ResultKeyOnChange3 = $Servlet->vars( 'Key3', 'abcd' );
    my $ResultKeyAfterChange3 = $Servlet->vars( 'Key3' );

    is( $ResultKey1, 'Value1', "$SubTestName - check return value for key1" );

    is( $ResultKey2, 'Value2', "$SubTestName - check return value for key2" );
    is( $ResultKey2OnChange, 'NewValue', "$SubTestName - check return value on change for key2 - is type check" );
    is( $ResultKey2AfterChange, 'NewValue', "$SubTestName - check new return value for key2" );

    is( $ResultKey3, 'Value3', "$SubTestName - check return value for key3" );
    is( $ResultKeyOnChange3, 'abcd', "$SubTestName - check return value on change for key3 - is type and value check" );
    is( $ResultKeyAfterChange3, 'abcd', "$SubTestName - check new return value for key3" );
}
#----------------------------------------------------------------------------------------
sub test_addVar_wrongType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopObject = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $ShopObject );
    $MockifyServlet->addVar( 'Key', 'Value', 'hashref' );
    my $Servlet = $MockifyServlet->getMockObject();

    throws_ok( sub { $Servlet->vars('Key','NotAHashValue'); },
              qr/Parameter\[0\] is not a HashRef: Method=DE_EPAGES::Presentation::Servlet::Permission->vars\(\?\?\?\); Value=NotAHashValue/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
              "$SubTestName - test the Error if the type is wrong"
    );
}
#----------------------------------------------------------------------------------------
sub test_addVar_wrongValue {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopObject = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $ShopObject );
    $MockifyServlet->addVar( 'Key', 'Value', {'string'=>'NewValue'} );
    my $Servlet = $MockifyServlet->getMockObject();

    throws_ok( sub { $Servlet->vars('Key','IncorrectString'); },
              qr/Parameter\[0\] unexpected value: ExpectedValue=NewValue; Method=DE_EPAGES::Presentation::Servlet::Permission->vars\(\?\?\?\); Value=IncorrectString/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
              "$SubTestName - test the Error if the value is wrong"
    );
}
#----------------------------------------------------------------------------------------
sub test_addVars {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopObject = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $ShopObject );
    $MockifyServlet->addVars( {
        'Key1' => 'Value1',
        'Key2' => 'Value2'
    } );
    my $Servlet = $MockifyServlet->getMockObject();

    my $ResultKey1 = $Servlet->vars( 'Key1' );
    my $ResultKey2 = $Servlet->vars( 'Key2' );


    is( $ResultKey1, 'Value1', "$SubTestName - checks the value of key1" );
    is( $ResultKey2, 'Value2', "$SubTestName - checks the value of key2" );
}
#----------------------------------------------------------------------------------------
# helper
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', {} );

    return $Shop;
}
__PACKAGE__->runScript();